{

  Copyright 1994-1995 by Salvatore Besso, mc8505@mclink.it

  This software is freeware.

  You are free to modify the source code for your personal use and to
  redistribute this software only if you leave the copyright notices
  unmodified everywhere in the code, also in the comments.

  In case of problems contact me via e-mail at address:

  mc8505@mclink.it

}

program DbDesc;

{ Prints Paradox 4 tables structure and indexes informations }

{
  Errorlevels returned:

  1  = File error (not found or other DOS error)
  2  = Paradox Engine error
  3  = Printer error
  99 = Command line parameter wrong or missing
}

uses
  Dos,Printer,
  Objects,
  PXEngine,OOPxEng;

const

  Version = '1.03';

  Engine  : PEngine   = NIL;
  DataBase: PDataBase = NIL;

  OnPrinter: Boolean = True;
  Path     : PathStr = '';

var
  I,J        : Integer;
  S          : String;
  Line,Spaces: String[128];
  Header     : array[1..10] of String;

procedure ReleaseEngine;

begin
  if Database <> NIL then Dispose (Database,Done);
  if Engine <> NIL then Dispose (Engine,Done)
end;

procedure ShutDown (Code: RetCode);

begin
  WriteLn (#7#13#10'Paradox Engine Database Framework error:');
  WriteLn (Engine^.GetErrorMessage (Code));
  ReleaseEngine;
  Halt (2)
end;

procedure OpenEngine;

begin
  Engine := New (PEngine,defInit (pxLocal));
  if Engine^.LastError <> PXSuccess then
    ShutDown (Engine^.LastError);
  DataBase := New (PDataBase,Init (Engine));
  if DataBase^.LastError <> PXSuccess then
    ShutDown (DataBase^.LastError)
end;

procedure PrinterError;

begin
  WriteLn (#7#13#10'Printer error');
  ReleaseEngine;
  Halt (3)
end;

function CenteredStr (S: String; Width: Integer): String;

begin
  CenteredStr := Copy (Spaces,1,(Width - Length (S)) div 2) + S
end;

procedure PrintStructure;

var
  Dir            : DirStr;
  Name           : NameStr;
  Ext            : ExtStr;
  DirInfo        : SearchRec;
  I              : Integer;
  TableDescriptor: PTableDesc;

procedure PrintInfo (P: PFieldDesc); far;

var
  S,T     : String;
  Len,Size: LongInt;

begin
  Str (P^.FldNum:0,T);
  S := Copy (T + Spaces,1,6) + ':  ' + Copy (P^.FldName + Spaces,1,27);
  case P^.FldType of
    fldChar  : T := 'Alpha';
    fldShort : T := 'Short';
    fldDate  : T := 'Date';
    fldDouble: T := 'Number';
    fldBlob  : T := 'Blob'
  end;
  S := S + Copy (T + Spaces,1,11);
  case P^.FldSubType of
    fldStNone   : T := 'None';
    fldStMoney  : T := 'Currency';
    fldStMemo   : T := 'Memo';
    fldStBinary : T := 'Binary';
    fldStFmtMemo: T := 'Fmt Memo';
    fldStOleObj : T := 'OLE';
    fldStGraphic: T := 'Graphic'
  end;
  S := S + Copy (T + Spaces,1,11);
  case P^.FldType of
    fldChar  : Len := P^.FldLen;
    fldShort : Len := 2;
    fldDate  : Len := 4;
    fldDouble: Len := 8;
    fldBlob  : Len := P^.FldLen
  end;
  Str (Len:0,T);
  S := S + Copy (T + Spaces,1,8);
  Size := Len;
  if P^.FldType = fldBlob then Inc (Size,10);
  Str (Size:0,T);
  S := S + T;
  {$I-}
  if OnPrinter then WriteLn (Lst,S) else WriteLn (S);
  {$I+}
  if IOResult > 0 then PrinterError
end;

procedure PrintIndexInfo (Dir: DirStr; Name: NameStr; TD: PTableDesc);

var
  DirInfo  : SearchRec;
  NumFlds  : Integer;
  S        : String;
  I        : Integer;
  P        : PFieldDesc;
  IndexName: String;
  CaseSen  : Boolean;
  FldArray : FieldNumberArray;
  IndexID  : Word;

begin
  { primary index info }
  NumFlds := DataBase^.GetNumPFields (Dir + Name);
  { if there is not a primary index, there are not secondary indexes also }
  if DataBase^.LastError = PXERR_TableNotIndexed then
    Exit
  else if DataBase^.LastError <> PXSuccess then
    ShutDown (DataBase^.LastError);
  Header[1] := '';
  Header[2] := CenteredStr ('Primary index for table ' + Name + '.DB',80);
  Header[3] := Copy (Line,1,80);
  Str (NumFlds:0,S);
  Header[4] := 'Number of fields: ' + S;
  Header[5] := '';
  Header[6] := 'Field #  Field Name';
  Header[7] := Copy (Line,1,80);
  for I := 1 to 7 do
  begin
    {$I-}
    if OnPrinter then WriteLn (Lst,Header[I]) else WriteLn (Header[I]);
    {$I+}
    if IOResult > 0 then PrinterError
  end;
  for I := 1 to NumFlds do
  begin
    P := PFieldDesc (TableDescriptor^.At (I - 1));
    Str (P^.FldNum:0,S);
    S := Copy (S + Spaces,1,6) + ':  ' + P^.FldName;
    {$I-}
    if OnPrinter then WriteLn (Lst,S) else WriteLn (S);
    {$I+}
    if IOResult > 0 then PrinterError
  end;
  { secondary indexes info }
  FindFirst (Dir + Name + '.XG?',Archive,DirInfo);
  while DosError = 0 do
  begin
    DataBase^.GetCompKeyInfo (Dir + DirInfo.Name,IndexName,NumFlds,CaseSen,
      FldArray,IndexID);
    if DataBase^.LastError <> PXSuccess then
      ShutDown (DataBase^.LastError);
    Header[1] := '';
    Header[2] := CenteredStr ('Secondary index for table ' + Name + '.DB',80);
    Header[3] := Copy (Line,1,80);
    Header[4] := 'Index name      : ' + IndexName;
    Str (IndexID:0,S);
    Header[5] := 'Index ID        : ' + S;
    Header[6] := 'Case sensitive  : ';
    if CaseSen then
      Header[6] := Header[6] + 'YES'
    else Header[6] := Header[6] + 'NO';
    Str (NumFlds:0,S);
    Header[7] := 'Number of fields: ' + S;
    Header[8] := '';
    Header[9] := 'Field #  Field Name';
    Header[10] := Copy (Line,1,80);
    for I := 1 to 10 do
    begin
      {$I-}
      if OnPrinter then WriteLn (Lst,Header[I]) else WriteLn (Header[I]);
      {$I+}
      if IOResult > 0 then PrinterError
    end;
    for I := 1 to NumFlds do
    begin
      P := PFieldDesc (TableDescriptor^.At (FldArray[I] - 1));
      Str (P^.FldNum:0,S);
      S := Copy (S + Spaces,1,6) + ':  ' + P^.FldName;
      {$I-}
      if OnPrinter then WriteLn (Lst,S) else WriteLn (S);
      {$I+}
      if IOResult > 0 then PrinterError
    end;
    FindNext (DirInfo)
  end
end;

begin { PrintStructure }
  FSplit (FExpand (Path),Dir,Name,Ext);
  if Name = '' then Name := '*';
  Ext := '.DB';
  FindFirst (Dir + Name + Ext,Archive,DirInfo);
  if DosError > 0 then
  begin
    WriteLn (#7#13#10'File not found or file error');
    ReleaseEngine;
    Halt (1)
  end;
  while DosError = 0 do
  begin
    Name := Copy (DirInfo.Name,1,Pos ('.',DirInfo.Name) - 1);
    TableDescriptor := DataBase^.GetDescVector (Dir + Name);
    if DataBase^.LastError <> PXSuccess then
      ShutDown (DataBase^.LastError);
    Header[1] := '';
    Header[2] := CenteredStr ('Table ' + Name + '.DB',80);
    Header[3] := Copy (Line,1,80);
    Header[4] := 'Field #  Field Name                 Type       ' +
      'Sub        Len     Size';
    Header[5] := Copy (Line,1,80);
    for I := 1 to 5 do
    begin
      {$I-}
      if OnPrinter then WriteLn (Lst,Header[I]) else WriteLn (Header[I]);
      {$I+}
      if IOResult > 0 then PrinterError
    end;
    TableDescriptor^.ForEach (@PrintInfo);
    PrintIndexInfo (Dir,Name,TableDescriptor);
    Dispose (TableDescriptor,Done);
    {$I-}
    if OnPrinter then Write (Lst,#12#13) else WriteLn;
    {$I+}
    if IOResult > 0 then PrinterError;
    FindNext (DirInfo)
  end
end;

procedure Usage;

begin
  WriteLn ('Usage: DbDesc [/NOPRINTER] [d:\path\]tablename[.DB]' +
    #13#10'              [>] [>>] [output device or file]' +
    #13#10'              (wildcards are OK)'#13#10);
  WriteLn ('Use /NOPRINTER to view the structures without printing.');
  WriteLn ('Use /NOPRINTER together with "> file/device" or ">> file" to ' +
    'redirect output'#13#10'to a file or device without printing.');
  Halt (99)
end;

begin { Main }
  WriteLn ('DbDesc ' + Version + ' - Prints Paradox 4 tables structure' +
    #13#10'by Salvatore Besso, mc8505@mclink.it'#13#10);
  if ParamCount = 0 then Usage;
  for I := 1 to ParamCount do
  begin
    S := ParamStr (I);
    for J := 1 to Length (S) do S[J] := Upcase (S[J]);
    if S[1] = '/' then
    begin
      if (Path <> '') or (S <> '/NOPRINTER') then
        Usage
      else OnPrinter := False
    end
    else if Path = '' then
      Path := ParamStr (I)
    else Usage
  end;
  if Path = '' then Usage;
  FillChar (Line[1],SizeOf (Line) - 1,'-');
  Line[0] := #128;
  FillChar (Spaces[1],SizeOf (Spaces) - 1,' ');
  Spaces[0] := #128;
  OpenEngine;
  PrintStructure;
  ReleaseEngine
end.
